@@ -500,6 +500,15 @@ sso: {
     fetchOpenIDConfig: function(wellKnownURL) {
         return APIXHRService("GET", API_PATH + "sso/fetch-openid-config", { wellKnownURL: wellKnownURL });
     },
+},
+
+alerting: {
+    getAlerting: function() {
+        return APIXHRService("GET", API_PATH + "alerting");
+    },
+    updateAlerting: function(alerting) {
+        return APIXHRService("PUT", API_PATH + "alerting", JSON.stringify(alerting));
+    },
 }
 
 };
