@@ -225,6 +225,7 @@ app.directive('codeEnvContainers', function (DataikuAPI, $rootScope) {
                 ['PYTHON36_SUPPORT', 'Install Python 3.6'],
                 ['PYTHON37_SUPPORT', 'Install Python 3.7'],
                 ['PYTHON38_SUPPORT', 'Install Python 3.8'],
+                ['AGENT_TOOL_MCP_SERVERS_SUPPORT', 'Agent tool MCP servers support'],
                 ['SYSTEM_LEVEL_CUDA_112_CUDNN_811', '[Deprecated] GPU support with CUDA 11.2'],
             ];
 
@@ -235,6 +236,7 @@ app.directive('codeEnvContainers', function (DataikuAPI, $rootScope) {
                 'Add support for Python 3.6',
                 'Add support for Python 3.7',
                 'Add support for Python 3.8',
+                'Add commands used by MCP servers (uvx, npx)',
                 '',
             ];
         }
