@@ -157,11 +157,24 @@
         }
         
         $scope.EMBED_DOCS_EXTRACTION_MODES = [ // keep in sync with EmbedDocumentsRecipeParams.java - ExtractionMode enum
-            {id: 'MANAGED_TEXT_ONLY', title: 'Text-only Extraction', description: "Extracts text from documents and uses headers to divide the content into meaningful extraction units. Supported file formats: PDF, PPTX, DOCX, HTML, TXT, MD."},
-            {id: 'MANAGED_VISUAL_ONLY', title: 'Visual Extraction', description: "Uses a Vision Language Model (VLM) to extract information from documents that may include text and visual elements like charts, graphics, and tables. Supported file formats: PDF, DOC, DOCX, PPT, PPTX, ODP, ODT, JPG, JPEG, PNG, TXT, MD."},
+            {id: 'MANAGED_TEXT_ONLY', title: 'Text-only Extraction', description: "Extracts text from documents and uses headers to divide the content into meaningful extraction units. Supported file formats: PDF, DOCX, PPTX, HTML, TXT, MD."},
+            {id: 'MANAGED_VISUAL_ONLY', title: 'Visual Extraction', description: "Uses a Vision Language Model (VLM) to extract information from documents that may include text and visual elements like charts, graphics, and tables. Supported file formats: PDF, DOC, DOCX, ODT, PPT, PPTX, ODP, JPG, JPEG, PNG, TXT, MD."},
             {id: 'CUSTOM_RULES', title: 'Custom rules', description: "Create your own rules for how text is extracted and choose which documents to apply them to. This gives you full control over the extraction process."},
         ];
 
+        $scope.VLM_EXTRACTION_PROMPTS = {
+            'SUMMARY_PROMPT': "Generate a concise summary, up to 500 words, derived from the screenshot(s) of document page(s) provided. \n"
+                + "Begin with a brief overview and highlight crucial words, facts, or concepts to enhance both semantic and keyword searchability.\n"
+                + "Omit any references to the original source.\n",
+
+            'FULL_CONTENT_PROMPT': "Extract the information from the screenshot(s) of document page(s) provided at the end, maintaining the original text without alteration. Follow these guidelines:\n"
+            + "* Use Markdown to format the text, including headers such as Title, Subtitle, and Main Sections, as well as any tables present in the document.\n"
+            + "* Describe any images and charts within the document screenshot(s), as the visual content cannot be directly extracted.\n"
+            + "* Exclude any hyperlinks, as they cannot be extracted from an image.\n"
+            + "* Ensure the text content remains unchanged.\n"
+            + "* Extract all text, including any footers or reference lists.\n",
+        }
+
         $scope.getActiveVLMfromLLMid = function(vlmId){
             return $scope.availableVLMs.find(llm => llm.id === vlmId);
 
@@ -181,13 +194,16 @@
                 "customNbPages": 1,
                 "customPagesOverlap": 0,
                 "useCustomPrompt" : false,
-                "customPrompt": "", // todo should keep the default prompt from backend
+                "customPrompt": $scope.VLM_EXTRACTION_PROMPTS["SUMMARY_PROMPT"],
             }
         }
         $scope.makeDefaultStructuredSettings = function(){
             return {
                 "splitUnit": "SECTION",
-                "maxSectionDepth": 6
+                "maxSectionDepth": 6,
+                "imageHandlingMode": "IGNORE",
+                "ocrEngine": "AUTO",
+                "ocrLanguages": "en",
             }
         }
 
@@ -458,7 +474,7 @@
 
         $scope.createCustomRulesFrom = function(extractionMode){
             if ($scope.retrievableKnowledge && $scope.recipe.params){
-                DataikuAPI.flow.recipes.nlp.getDefaultEmbedDocumentRules(extractionMode.id, $scope.retrievableKnowledge.embeddingLLMId, $scope.recipe.params.defaultVlmId, $stateParams.projectKey).success(function(data) {
+                DataikuAPI.flow.recipes.nlp.getDefaultEmbedDocumentRules(extractionMode.id, $scope.retrievableKnowledge.embeddingLLMId, $scope.recipe.params.defaultVlmId, $stateParams.projectKey, $scope.recipe.params.defaultImageHandlingMode).success(function(data) {
                     if (data) { 
                         $scope.recipe.params.rules = data.rules;
                         Object.assign($scope.recipe.params.allOtherRule, data.allOtherRule); // only copy properties without changing reference
@@ -467,7 +483,17 @@
                 }).error(setErrorInScope.bind($scope));
             }
         }
-            
+        $scope.isMetadataDatasetFilled = function(){
+            return $scope.recipe.inputs['metadata_dataset'].items.length > 0;
+        }
+
+        $scope.overrideCustomPromptWith = function (vlmSettings, promptVersion) {
+
+            if(!promptVersion || !$scope.VLM_EXTRACTION_PROMPTS[promptVersion]){
+                return;
+            }
+            vlmSettings.customPrompt = $scope.VLM_EXTRACTION_PROMPTS[promptVersion];
+        }
     });
  
 
