@@ -408,8 +408,14 @@ app.controller("OutcomesViewController", function($scope, $controller, $statePar
 
     $scope.reportRange = {from : 0, to : 0}
     $scope.displayedColumns = [];
+    $scope.displayedRows = [];
     $scope.scenarioOwners = [];
-    $scope.scenarioOwnersFilter = null;
+    $scope.automationMonitoringScenarioOutcomeFilter = {
+        owner: null,
+        status: 'ALL',
+        tags: []
+    };
+    $scope.allTags = [];
 
     function deduplicateArrayOfArrays(arrayWithDuplicates) {
       let seen = new Set();
@@ -432,6 +438,7 @@ app.controller("OutcomesViewController", function($scope, $controller, $statePar
         DataikuAPI.scenarios.getOutcomes(rangeFromTs.format(backendFormat), rangeToTs.add(1, 'days').format(backendFormat), $scope.projectKey, $scope.scenarioId).success(function(data){
             $scope.outcomes = data;
             if ($scope.outcomes && $scope.outcomes.rows) {
+                $scope.scenarioOwners = [];
                 $scope.outcomes.rows.forEach(
                     function(item, index) {
                         $scope.scenarioOwners.push([item.info.login, item.info.runAsDisplayName]);
@@ -457,6 +464,8 @@ app.controller("OutcomesViewController", function($scope, $controller, $statePar
 //                $scope.reportRange.to = $scope.outcomes.key.to < $scope.reportRange.to ? $scope.outcomes.key.to : $scope.reportRange.to;
 //            }
             $scope.refreshDisplayedColumns();
+            $scope.refreshDisplayedRows();
+            $scope.fetchTags();
         }).error(setErrorInScope.bind($scope))
         .finally(() => {$scope.loading = false;});       
     };
@@ -468,9 +477,24 @@ app.controller("OutcomesViewController", function($scope, $controller, $statePar
         if ( nv == null || ov == nv ) return;
         $scope.refreshOutcomes();
     });
-    $scope.scenarioOwnerChanged = function(newScenarioOwner) {
-        $scope.scenarioOwnersFilter = newScenarioOwner;
+
+    $scope.onFilterUpdated = function onFilterUpdated() {
         $scope.refreshDisplayedColumns();
+        $scope.refreshDisplayedRows();
+    };
+
+    $scope.fetchTags = function fetchTags() {
+        for (const row of $scope.displayedRows) {
+            const taggableObject = {
+                type: row.target.type,
+                projectKey: row.target.projectKey,
+                id: row.target.scenarioId
+            };
+            DataikuAPI.taggableObjects.getMetadata(taggableObject).success((result) => {
+                row.tags = result.tags;
+                $scope.allTags = Array.from(new Set([...$scope.allTags, ...result.tags]));
+            });
+        }
     }
 
     $scope.refreshDisplayedColumns = function() {
@@ -482,20 +506,64 @@ app.controller("OutcomesViewController", function($scope, $controller, $statePar
             $scope.displayedColumns = $scope.outcomes.columns.filter(function(column) {return column.date >= startDate && column.date <= endDate});
         }
     };
+
+    $scope.refreshDisplayedRows = function refreshDisplayedRows() {
+        if ( $scope.outcomes == null || $scope.outcomes.rows == null || $scope.reportRange == null ) {
+            $scope.displayedRows = null;
+        } else {
+            $scope.displayedRows = $scope.scenarioSearch($scope.outcomes.rows, $scope.displayedColumns);
+        }
+    }
+
     $scope.$watch('reportRange', function(nv) {
         if ( nv == null ) return;
         $scope.refreshDisplayedColumns();
     }, true);
 
     $scope.scenarioQuery = null;
-    $scope.scenarioSearch = function(row) {
-        if ($scope.scenarioOwnersFilter != null) {
-            return searchInTargetAndInfo(row, $scope.scenarioQuery) && row.info.login == $scope.scenarioOwnersFilter;
-        } else {
-            return searchInTargetAndInfo(row, $scope.scenarioQuery);
-        }
+    $scope.scenarioSearch = function scenarioSearch(rows, columns) {
+        if (!rows || !rows.length || (!$scope.scenarioQuery && !$scope.automationMonitoringScenarioOutcomeFilter)) return rows;
+        return rows.filter(row => {
+            if ($scope.automationMonitoringScenarioOutcomeFilter) {
+                return searchInTargetAndInfo(row, $scope.scenarioQuery) && filterScenarioSearchResults(row, columns);
+            } else {
+                return searchInTargetAndInfo(row, $scope.scenarioQuery);
+            }
+        });
     };
 
+    /**
+     * 
+     * @param {object[]} row 
+     * @param {object[]} columns 
+     * @returns 
+     */
+    function filterScenarioSearchResults(row, columns) {
+        const { owner, status, tags } = $scope.automationMonitoringScenarioOutcomeFilter;
+        if (owner && row.info.login != owner) {
+            return false;
+        }
+        const scenarioContainsTag = tags && tags.length > 0 && (!row.tags || !row.tags.some(tag => tags.includes(tag)));
+        if (scenarioContainsTag) {
+            return false;
+        }
+
+        const automationId = row.uniqueId;
+        switch (status) {
+            case 'failed':
+            case 'warning':
+            case 'aborted':
+                return columns.some(col => $scope.getCellGlobalOutcome(col.actions[automationId]) === status);
+            case 'success':
+                // None of the actions is failed
+                return columns.every(col => $scope.getCellGlobalOutcome(col.actions[automationId]) !== 'failed');
+            case 'all':
+            default:
+                return true;
+
+        }
+    }
+
     $scope.brushChanged = function() { $scope.$digest(); };
 
     $scope.hover = function(evt, column, row, localScope) {
@@ -552,9 +620,6 @@ app.directive('outcomeCells', function($filter, $timeout) {
                 var rows = $scope.$eval(attrs.ngModel);
                 var columns = $scope.displayedColumns;
                 if (rows == null || columns == null) return;
-                if ($scope.scenarioSearch) {
-                    rows = $filter('filter')(rows, $scope.scenarioSearch);
-                }
                 element.empty();
                 hoverCells.splice(0,hoverCells.length);
                 rows.forEach(function(row) {
