@@ -160,8 +160,10 @@
 
                 addBigChartClass(false); // remove class in case user is switching from a big chart
                 scope.bigChart = false;
+                scope.chartHeaderLabel = translate('CHARTS.HEADER.EXPAND', 'Expand chart');
                 scope.bigChartSwitch = function() {
                     scope.bigChart = !scope.bigChart;
+                    scope.chartHeaderLabel = scope.bigChart ? translate('CHARTS.HEADER.REDUCE', 'Reduce chart') : translate('CHARTS.HEADER.EXPAND', 'Expand chart');
                     scope.isBigChartSwitch = true;
                     $('.graphWrapper').fadeTo(0, 0);
                     addBigChartClass(scope.bigChart);
