@@ -96,6 +96,7 @@ class ModelParamsLoader:
         self.torch_dtype = model_settings.get("dtype")
         self.tensor_parallel_size = model_settings.get("tensorParallelSize")
         self.pipeline_parallel_size = model_settings.get("pipelineParallelSize")
+        self.enable_expert_parallelism = model_settings.get("enableExpertParallelism")
 
         self._apply_overrides()
 
@@ -141,11 +142,12 @@ class ModelParamsLoader:
             )
             params["tensor_parallel_size"] = self.tensor_parallel_size
 
-            # Explicitly use Ray for multi-GPUs because Multiprocessing does not work out of the box:
-            # - Spawned Process is not cleaned and remains on the GPU once we close the kernel (local exec only).
-            #   Tricky to handle so we override the default value to use Ray as before
+            # - When the process tree is killed by the Java backend, Ray leaves Dashboard and RuntimeAgent processes alive for one minute before they die on their own
+            #   Multiprocessing behaves better: all processes are killed on the spot
+            # - Ray processes can be killed when engine V0 is started in background thread, if the thread pool executor is shutdown at the end, so multiprocessing seems more robust
+            # More details in https://github.com/dataiku/dip/pull/38752
             if self.tensor_parallel_size > 1:
-                params["distributed_executor_backend"] = "ray"
+                params["distributed_executor_backend"] = "mp"
         if self.pipeline_parallel_size is not None:
             logger.info(
                 "Pipeline parallel: {pipeline_parallel_size}".format(
@@ -154,11 +156,15 @@ class ModelParamsLoader:
             )
             params["pipeline_parallel_size"] = self.pipeline_parallel_size
 
-            # Explicitly use Ray for multi-GPUs because Multiprocessing does not work out of the box:
-            # - Spawned Process is not cleaned and remains on the GPU once we close the kernel (local exec only).
-            #   Tricky to handle so we override the default value to use Ray as before
+            # Bug with vllm: engine V1 + PP requires distributed_executor_backend to be explicitly set (either to "ray" or "mp")
+            # - When the process tree is killed by the Java backend, Ray leaves Dashboard and RuntimeAgent processes alive for one minute before they die on their own
+            #   Multiprocessing behaves better: all processes are killed on the spot
+            # - Ray processes can be killed when engine V0 is started in background thread, if the thread pool executor is shutdown at the end, so multiprocessing seems more robust
+            # More details in https://github.com/dataiku/dip/pull/38752
             if self.pipeline_parallel_size > 1:
-                params["distributed_executor_backend"] = "ray"
+                params["distributed_executor_backend"] = "mp"
+        if self.enable_expert_parallelism is not None:
+            params["enable_expert_parallel"] = self.enable_expert_parallelism
 
         return params
 
@@ -186,7 +192,8 @@ class ModelParamsLoader:
 
     def _dtype(self) -> Optional[Union[torch.dtype, str]]:
         if (
-                hasattr(self.model_config, "torch_dtype")
+                torch.cuda.is_available()
+                and hasattr(self.model_config, "torch_dtype")
                 and self.model_config.torch_dtype == torch.bfloat16
                 and not is_bfloat16_supported_with_cuda()
         ):
@@ -430,7 +437,7 @@ class MistralNemo12BModelParamsLoader(ModelParamsLoader):
         # - 2x24GB GPUs
         # - 1x24GB GPU with 4bit quantization.
         if self.is_24gb_gpu or not self.with_quantization:
-            reduced_context_len = 64000
+            reduced_context_len = 32000
             logger.warning(
                 textwrap.dedent(
                     f"""
